#define SUNLIMIT -8 /* If the sun is above the horizon you can't see anything */
#define SATLIMIT -3 /* If the sun is below the horizon for the sat it isn't lit */
#define MINELEV   5.  /* Satellite has to be at least this high */
#define DAYS	14.   /* Go for 2 weeks by default */

/*
DRIVE1.C
by Paul S. Hirose, 19 April 1989
main module for the SEESAT satellite tracking program.
*/

/* Breakpoints 0 - 99 reserved for this module. */


/* library functions, #defines, and typedefs used by this module:
	exit fopen FILE NULL printf strcmp
*/
#include <stdio.h>	/* FILE, NULL */
extern int strcmp();
extern FILE *fopen();

/* DRIVER must be defined to gain access to material in SEESAT.H
shared by DRIVE1 & DRIVE2 */
#define DRIVER

#include "seesat.h" /* global header */

/*############################## DATA ##############################*/
/* function pointer.  Controls the orbital model, e.g. sgp4(), to use
for generating predictions. */

static void (*model)() = sgp4;

/*#################### functions in this module ####################*/
static void load(), init();
static double time();

/*############################## CODE ##############################*/

main(argc, argv)
int argc;
char **argv;
{
	double	start, stop, step,	/* entered by user */
		tpho,		/* time-of-day at beginning of prediction run,
				min */
		zd;		/* time zone description (GMT - local) */

	int	iflag, *iptr;

	char	*cptr,
		date[12];	/* date string e.g. "29 FEB 1987" */


	stop = 0.;
	step = 0.;

	ETEST((0, 1, 3, &PI));	/* dummy call to get control of brkpt */

	printf("SEESAT %s by Paul S. Hirose\n", VERS);

	init();		/* initialize global constants */
	zd = topos();	/* get observer's coordinates, set time zone */
	iflag = 1;	/* = orbital model not initialized yet */

	if (argc >= 2)
	/* load orbital element file specified on cmd line */
		load(argv[1]);

	/* from here on, argv is used to point to items on SEESAT's
	command line.  Initialize it to point to null string. */
	*argv = NULL;

	mjog();		/* display available commands */

	while (1) {	/* main loop */

		FTEST((2, 4, 2, &start, &step, &tpho, &stop));

		/* get the next command and associated args */
		argv = tok(argv);
		cptr = *argv;

		/* figure out what was typed, take appropriate action */
		if (!strcmp("START", cptr)) {
			tpho = atomin(argv[1]);
			start = time(&argv, &iptr) + zd;
			tokdat(iptr, date);
		} else if (!strcmp("STOP", cptr))
			stop = time(&argv, &iptr) + zd;
		else if (!strcmp("STEP", cptr)) {
			step = atomin(argv[1]);
			argv += 2;
		} else if (!strcmp("RUN", cptr)) {
			double tp, tph;
			int iflag2;
			int passflag;
			int sunelevation;

			if (stop == 0.)
				stop = start + DAYS * 24. * 60.;
			if (step == 0.)
				step = 20. / 60.;

			++argv;
			if (stop < start) {
				printf("STOP IS EARLIER THAN START\n");
				continue;
			} printf("%s  %s\n", name, date);
			tp = start;
			tph = tpho;
			iflag2 = 1;
			passflag = 0;  /* Start out not in a pass */

			do {
				sunelevation = dusk(tp);
				if (sunelevation <= SUNLIMIT)
				{
					(*model)(&iflag, tp - epoch);
					if (xyztop(&iflag2, tp) &&
					(elsusa >= SATLIMIT)    &&
					(azel.phi * RA2DE >= MINELEV))
					{
					/* sat is above horizon */
					/* sun is above horizon for sat */
					/* sat is far enough above horizon */
					
						if (passflag == 0)
						{
							printf("\nStarting pass on ");
							printjuldat(tp-zd);
							printf(", ");
							printf("sun elevation %d degrees.\n", sunelevation);
                        				printf("  TIME   AZ EL   R.A.    DEC      RNG SUN    ALT LAT  LON\n");
						}
						passflag = 1;
						prnval(tph);
					}
					else
					{
						if (passflag == 1)
						{
							printf("Ending pass on ");
							printjuldat(tp-zd);
							printf(", ");
							printf("sun elevation %d degrees.\n", sunelevation);
							passflag = 0;
						}
					}
				}
				else
				{
					if (passflag == 1)
					{
						printf("Ending pass on ");
						printjuldat(tp-zd);
						printf(", ");
						printf("sun elevation %d degrees.\n", sunelevation);
						passflag = 0;
					}
					/* tell xyztop we may be jumping to a new time */
					iflag2 = 1;
				}

				tp += step;
				tph += step;
			} while (tp <= stop);
		} else if (!strcmp("SUN", cptr))
			printf("SUN ELEVATION = %d\n",
			 dusk(time(&argv, &iptr) + zd));
		else if (!strcmp("PARA", cptr))
			printf("PARALLACTIC ANGLE = %d\n",
			 parall(time(&argv, &iptr) + zd));
		else if (!strcmp("LOAD", cptr)) {
			load(argv[1]);
			argv += 2;
			iflag = 1;
		} else if (!strcmp("?", cptr)) {
			++argv;
			mjog();		/* display the memory jogger */
		} else if (!strcmp("EXIT", cptr))
			exit(0);
		else {
			printf("%s: UNRECOGNIZED COMMAND\n", cptr);
			mjog();		/* print memory jogger */
			*argv = NULL;		/* clear command line */
}	}	}

/*------------------------------------------------------------------*/

static void
load(name)
char *name;
{
/* Opens file "name", loads the global orbital element variables from
the file. */

	FILE *orbel;

	if ((orbel = fopen(name, R)) == NULL) {
		printf("can't open %s\n", name);
		return;
	}
	/* set global orbital element variables to values from file */
	switch (getel(orbel)) {
	case -1:
		printf("error in file, can't use\n");
		return;
	case 0:
		printf("orbital elements loaded ok\n");
		return;
	case 1:
		printf("deep space object; can't compute orbit\n");
}	}

/*------------------------------------------------------------------*/

static void
init()
{
	/* initialize global constants */

	static double
		temp,
		qo = 120.0,
		so = 78.0;

	PIO2 = PI * .5;
	TOTHRD = 2. / 3.;
	TWOPI = PI + PI;
	X3PIO2 = 1.5 * PI;
	DE2RA = PI / 180.;
	RA2DE = 180. / PI;

	/* earth physical constants */
	XJ3 = -.253881e-5;	/* 3rd & 4th gravitational */
	XJ4 = -1.65597e-6;	/* zonal harmonics */
	temp = AE * AE;
	ck2 = .5 * XJ2 * temp;
	ck4 = -.375 * XJ4 * temp * temp;

	/* Density parameters */
	temp = (qo - so) * AE / XKMPER;
	qoms2t = temp * temp * temp * temp;
	s = AE * (1.0 + so / XKMPER);
}

/*------------------------------------------------------------------*/

static double
time(pargv, iarray)
char ***pargv;
int **iarray;
{

/* Converts a time/day/month/year string on command line to epoch, returns
that value.  As a side effect it sets *iarray to the address of an array
containing the year, month, & day.  The pargv variable is a pointer to argv
in main(); argv is passed in here by reference so (as another side effect)
it can be stepped forward to the next command on the cmd line. */

	double temp;
	int *iptr;
	char **argv;

	argv = *pargv + 2;	/* points to day string in main() */
	temp = atomin(*(argv - 1));	/* temp = time of day */
	iptr = dday(&argv);		/* iptr[] = {day, mo, yr} */
	*pargv = argv;		/* return argv to main() */
	*iarray = iptr;		/* return iptr to main() */
	return julday(iptr[2], iptr[1], iptr[0]) * XMNPDA + temp;
}
