/*
DRIVE2.C
Paul S. Hirose, 15 April 1989
Functions supporting main() of SEESAT satellite track program.
*/

/* Reserved software test points 100 - 199 */

/* library functions used by this module:
	atof fabs getchar isdigit isspace printf sprintf
	strcpy strlen strncpy toupper
In addition, NULL must be defined. */

/* library function declarations */
#define NULL 0
extern int getchar(), isdigit(), isspace(), strlen(), toupper();
extern char *strcpy(), *strncpy();
extern double atof(), fabs();

#define DRIVER		/* must be defined to pull in the necessary
			data from global header */
#include "seesat.h" /* global header */


/* normally 0.  Non-zero will make static functions extern for testing */
#define NSTAT 0

#if NSTAT
#define STATIC		/* precedes static func definitions */
#define SC extern	/* precedes static func declarations */

#else
#define STATIC static
#define SC static
#endif

/*###################### FUNCTIONS THIS FILE ######################*/

SC char *s_in();	/* input a string with prompt */
SC char *stoup();	/* change string to upper case */

/*############################## CODE ##############################*/

STATIC char *
s_in(prompt, buffer)
char *prompt, *buffer;

/* Prints the prompt string on console, puts typed string in buffer[]. 
Programmer must ensure buffer is big enough to hold whatever will be typed,
plus a null terminator.  Backspacing will correct typing mistakes.  Will
not allow backspacing past the start of buffer.  Returns buffer. */

{
	static char c, *ptr, last;

	/* Get last char in prompt string.  It's needed to restore screen
	if user tries to backspace too far. */
	for (ptr = prompt; *ptr++; )	/* find char after null */
		;
	last = *(ptr - 2);		/* fetch char before the null */

	printf("%s", prompt);
	ptr = buffer;
	while ((c = getchar()) != '\n')
		if (c == '\b')
			if (ptr == buffer)
				printf("%c", last);
			else
				--ptr;
		else
			*ptr++ = c;

	*ptr = '\0';
	return buffer;
}

/*================================================================*/

void
mjog()
{
	printf("START STOP STEP RUN SUN PARA LOAD EXIT\n");
	printf("? will get this display\n");
}

/*================================================================*/

STATIC char *
stoup(string)
char *string;

/* Converts "string" to all upper case, returns "string". */

{	char *ptr = string;

	while (*ptr = toupper(*ptr))
		++ptr;

	return string;
}

/*==================================================================*/
	
char **
tok(argv)
char **argv;
{
	int notok;		/* == 1 if we're not in a token */
	static char
		c,
		*cptr,
		buffer[LCLIN],		/* command line buffer */
		*tokens[NTOK],		/* pointers to cmd line tokens */
		**tokptr;		/* points into tokens[] */

	if (*argv == NULL) {
	/* buffer is empty; display '>', get more input, tokenize it */
		prompt:
		if (!strlen(stoup(s_in(">", buffer))))
			goto prompt;		/* empty line typed */

		cptr = buffer - 1;
		tokptr = tokens;
		notok = 1;
		while (c = *++cptr)
			if (isspace(c)) {
				notok = 1;
				*cptr = '\0';
			} else
				if (notok) {
				/* first char of a token */
					*tokptr++ = cptr;
					notok = 0;
				}
		*tokptr = NULL;		/* terminate tokens[] */
		argv = tokens;
	} return argv;
}

/*==================================================================*/

double
din(str)
char *str;
{
	char buffer[30];

	printf("[%s]:", str);
	if (strlen(s_in(" ", buffer)))
		return atof(buffer);
	return atof(str);
}

/*==================================================================*/

void
prnval(tph)
double tph;
{
	int sign;

	/* time, azimuth, elevation. */
	degdms(4, tph / 60.0);
	printf("%s %3d %2d  ", dmstim(),
	 (int) (azel.lambda * RA2DE + .5), (int) (azel.phi * RA2DE + .5));

	/* Right Ascension, declination. */
	degdms(2, radec.lambda / TWOPI * 24.0);
	printf("%2sh%2sm ", timstr.deg, timstr.min);
	degdms(1, radec.phi * RA2DE);
	printf("%3s %2s'  ", timstr.deg, timstr.min);

	/* slant range, sun elevation at satellite */
	printf("%5d %3d  ", (int) (radec.r * XKMPER + .5), elsusa);

	/* altitude */
	printf("%5d ", (int) (latlon.r * XKMPER + .5));

	/* latitude */
	if (latlon.phi >= 0.0)
		sign = 0;
	else
		sign = 1;
	printf("%2d%c ", (int) (fabs(latlon.phi * RA2DE) + .5),
	 sign ? 'S' : 'N');

	/* longitude */
	if (latlon.lambda >= 0.0)
		sign = 0;
	else
		sign = 1;
	printf("%3d%c\n", (int) (fabs(latlon.lambda * RA2DE) + .5),
	 sign ? 'W' : 'E');
}

/*==================================================================*/

double
atomin(string)
char *string;

{	double time;
	char *ptr, c;
	int i, n;
	static char buf[5];

	ptr = string;

	/* point to the colon (or just after last digit if no colon) */
	while ((c = *ptr) != ':' && c)
		++ptr;

	/* process seconds */
	if (c) {
		/* string had a colon */
		strncpy(buf, ptr + 1, 3);
		time = atof(skip0(buf)) / 60.0;
	} else
		time = 0;

	/* move hrs & mins digits to buf[], padding with leading
	zeros as necessary to make a 4-char "hhmm" string */

	i = n = 4 - (int) (ptr - string);	/* # of padding 0's needed */
	if (n < 0)
	/* defensive coding in case string has too many digits */
		i = n = 0;

	while (n--)
		buf[n] = '0';		/* pad with leading zeros */
	strncpy(buf + i, string, 5 - i);


	/* minutes */
	time += atof(skip0(buf + 2));
	/* hours */
	buf[2] = '\0';
	time += atof(skip0(buf)) * 60.0;

	return time;
}

/*------------------------------------------------------------------*/

void
tokdat(dmy, dest)
int *dmy;
char *dest;
{
	static char *month[] = {"", "JAN", "FEB", "MAR", "APR", "MAY", "JUN",
				"JUL", "AUG", "SEP", "OCT", "NOV", "DEC"};

	sprintf(dest, "%d %s %d", dmy[0], month[dmy[1]], dmy[2]);
}

/*------------------------------------------------------------------*/

void
degdms(pre, x)
int pre;
double x;

{	double d;
	int i, sign;

	if (x >= 0)
		sign = 0;
	else
		sign = 1;
	x = fabs(x);

	/* compute value of least significant digit
	specified by pre */
	if (pre <= 2) {
		if (pre == 1)
			d = .16666666666666667;		/* 1/6 */
		else
			d = .16666666666666667e-1;	/* 1/60 */
	} else
		d = POW(10.0, (double) (4 - pre)) / 3600.0;

	/* Round x to nearest multiple of d. */

	x = dint(x / d + .5);
	x *= d;

	/* x may not be an exact multiple of d.  This will cause difficulties
	if (for example) we want rounding to nearest 10 min. but x is
	equivalent to 34 deg. 49.9999999 sec.  Because dint() truncates the
	fractional part of its argument, we will get 49 minutes in this
	example rather than the obviously correct 50 unless I do some
	defensive coding.

	The trick is to add some small quantity to x, enough to roll it
	over if it's low, but not so much that the wrong number comes
	out. */

	switch (pre) {
	case 1:
	case 2:
		d = .83333333333333333e-2;	/* 1/2 minute */
		break;
	case 3:
	case 4:
		d = .13888888888888889e-3;	/* 1/2 second */
		break;
	default:
		d = 0.0;

	} x += d;

	/* get degrees */
	i = x;
	if (sign)
		sprintf(timstr.deg, "-%d", i);
	else
		sprintf(timstr.deg, "%d", i);

	/* get minutes */
	x = (x - i) * 60.0;	/* x = minutes remainder */
	i = x;
	sprintf(timstr.min, "%d", i);

	if (pre < 3) return;

	/* get seconds */
	x = (x - i) * 60.0;

	if (pre < 5) {
		sprintf(timstr.sec, "%d", (int) x);
		return;

	} sprintf(timstr.sec, "%.*f", pre - 4, x);
}

/*==================================================================*/

char *
dmstim()

{	static char buf[8];
	char *cptr = buf;
    int temp;

    /* Roundoff at 24 hr -AD */
    temp = atoi(timstr.deg);
    temp %= 24;
    /* itoa(temp, timstr.deg, 10);       --ss */
    sprintf(timstr.deg, "%d", temp);  /* --ss */
    
	if (strlen(timstr.deg) == 1)
		*cptr++ = '0';
	strcpy(cptr, timstr.deg);

	cptr = buf + 2;
	if (strlen(timstr.min) == 1)
		*cptr++ = '0';
	strcpy(cptr, timstr.min);

	buf[4] = ':';
	cptr = buf + 5;
	if (strlen(timstr.sec) == 1)
		*cptr++ = '0';
	strcpy(cptr, timstr.sec);

	return buf;
}

/*==================================================================*/

char *
skip0(string)
char *string;
{	for ( ; *string == '0'; ++string)
		;
	return string;
}

/*==================================================================*/

int *
dday(pargv)
char ***pargv;
{
	static int j[3];
	int n;
	char *cptr, **argv = *pargv;

	for (n = 0; n != 3; ++n, ++argv) {
		cptr = *argv;
		if (!isdigit(*cptr))
		/* 1st char of next token isn't digit */
			break;
		j[n] = atoi(cptr);
	}
	*pargv = argv;
	return j;
}
