/*
READEL.C
23 April 1989 by Paul S. Hirose
Handles orbital element files for the SEESAT satellite prediction program.
*/

/* Software test points 400 through 499 reserved for this module */

/* library functions & preprocessor symbols used in this module:
	atof atoi cos EOF FILE getc isdigit isspace strncpy */


#include "seesat.h" /* global header */


#define NSTAT 0		/* normally 0.  Non-zero makes static functions
			extern, for testing */


/*######################## LOCAL FUNCTIONS ########################*/

#if NSTAT
/* define & declare all functions in this file as extern */
#define STATIC
#define SC extern

#else
#define STATIC static
#define SC static
#endif

SC int getlin();	/* move a line from file to buffer */
SC int csum();		/* checksums a line of data */
SC double epjd();	/* converts NASA/NORAD Epoch to Julian Day */
SC void nullsp();	/* replace spaces in buffer with nulls */
SC double angle();	/* extract angle from buffer */
SC char *sknull();	/* skip leading nulls */
SC double sci();	/* returns value of scientific notation #'s */

/*############################## CODE ##############################*/

int
getel(fp)
FILE *fp;	/* orbital element file */
{
	double ao, a1, delo, del1, temp;
	int i;
	char c, line2[70], line3[70];

	/* Get name */
	if (!getlin(name, fp, 12))
		return -1;	/* end of file */

	/* Get first data line */
	if (!(i = getlin(line2, fp, 70)))
		return -1;	/* end of file */

	/* Line we've gotten should be 69 chars long begin, with "1 ",
	and pass checksum. */
	if (i != 69 || line2[0] != '1' || line2[1] != ' '
	 || csum(line2))
		return -1;	/* error */

	/* get second data line */
	if (!(i = getlin(line3, fp, 70)))
		return -1;	/* end of file */

	/* check for valid line. */
	if (i != 69 || line3[0] != '2' || line3[1] != ' '
	 || csum(line3))
		return -1;	/* error */

	/* print both lines at console */
	printf("%s\n%s\n", line2, line3);

	/* replace spaces in data lines with nulls */
	nullsp(line2);
	nullsp(line3);

	xmo = angle(line3 + 43);
	xnodeo = angle(line3 + 17);
	omegao = angle(line3 + 34);
	xincl = angle(line3 + 8);
	FTEST((400, 4, 3, &xmo, &xnodeo, &omegao, &xincl));

	/* put a decimal point in front of eccentricity, decode it */
	line3[25] = '.';
	eo = atof(line3 + 25);

	temp = TWOPI / (XMNPDA * XMNPDA);

	/* Make sure mean motion is null-terminated, since rev. no.
	may immediately follow. */
	line3[63] = '\0';
	xno = atof(sknull(line3 + 52)) * temp * XMNPDA;
	xndt2o = atof(sknull(line2 + 33)) * temp;
	xndd6o = sci(line2 + 44) * temp / XMNPDA;

	bstar = sci(line2 + 53) * AE;
	epoch = epjd(line2 + 18);
	ETEST((402, 5, 3, &eo, &xno, &xndt2o, &xndd6o, &bstar));
	FTEST((403, 1, 3, &epoch));

	/* initialize direction cosines to precess R.A./dec. */
	inpre(epoch);

	/* Determine if satellite is a deep space object */
	a1 = POW(XKE / xno, TOTHRD);
	temp = cos(xincl);
	temp = 1.5 * ck2 * (3. * temp * temp - 1.) /
	  POW(1. - eo * eo, 1.5);
	del1 = temp / (a1 * a1);
	ao = a1 * (1. - del1 * (.5 * TOTHRD + del1 *
	  (1. + 134. / 81. * del1)));
	delo = temp / (ao * ao);

	if (TWOPI / (xno / (1. + delo)) / XMNPDA >= .15625)
		return 1;	/* deep space */
	return 0;		/* near earth */
}

/*------------------------------------------------------------------*/

STATIC int
getlin(dest, fp, size)
int size;	/* must be at least 2 */
char *dest;
FILE *fp;

/* Copy a line from "fp" into "dest[]".  "Size" is length of dest[].  Chars
are copied to till size - 1 have been copied, or '\n' found (the '\n' won't be
sent to dest[]), or EOF.  Leading white space chars (including blank lines)
are stripped (i.e., dest[0] will not be white space).  If size - 1 chars were
successfully copied, input file is scanned till '\n' or EOF is found.  This
ensures getlin() starts on a fresh line if it's called again.

String copied to dest[] will be null terminated.  Returns # of chars placed in
dest[] (final null not counted), or 0 if end of file reached before
"count" - 1 were copied.  Getlin() will never return 0 unless EOF is found.
The "size" arg must be at least 2. */

{
	int c, i;

	i = size - 1;	/* i = # of chars desired */

	/* go past leading white space */
	while (isspace(c = getc(fp)))
		;

	/* Move chars to buffer till we we reach end of line, or
	have gotten # requested, or end of file. */

	do {
		if (c == EOF)
			break;
		*dest++ = c;
	} while (--i && (c = getc(fp)) != '\n');

	/* At this point, i is how many chars we've fallen short */
	/* Terminate string with null. */
	*dest = 0;

	if (!i)
		/* got requested # of chars, continue scanning till
		end of line or EOF.  Use do-loop since c can't be
		'\n' or EOF yet (because we DID get requested #
		of chars). */
		do
			c = getc(fp);
		while (c != '\n' && c != EOF);

	else if (c == EOF)
		/* EOF before requested # of chars found */
		return 0;

	/* exit here if we didn't hit EOF */
	return size - i - 1;
}

/*------------------------------------------------------------------*/

STATIC int
csum(line)
char *line;

/* Does a checksum modulo 10 on the given line.  Digits = their
value, '-' = 1, all other chars = 0.  Returns 0 if ok. */

{
	char c;
	int i = 0;	/* checksum accumulator */
	int count = 69;		/* length of data line */

	/* accumulate checksum from all but the last char in the line,
	which is the desired checksum */

	while (--count) {
		if (isdigit(c = *line++))
			i += c - '0';	/* turn ASCII to number */
		else if (c == '-')
			++i;		/* minus = 1 */
		/* all other chars = 0 */
	}

	/* Convert accumulated # to mod 10, subtract desired sum */
	return (i % 10 - *line + '0');
}

/*==================================================================*/

/* Given pointer to the orbital elements Epoch field, returns
Julian Day (expressed in minutes).  Tens and hundreds Days digits
may be omitted; if so, their positions must contain '0' or '\0'.
Format of Epoch field:  YYDDD.dddddddd */

STATIC double
epjd(buf)
char *buf;
{	
	double day;
	int year;
	char *cptr;

	day = atof(skip0(sknull(buf + 2)));

	buf[2] = '\0';			/* day 100s digit = null */
	year = atoi(buf);

	return (julday(year + 1899, 12, 31) + day) * XMNPDA;
}

/*==================================================================*/

STATIC void
nullsp(buf)
char *buf;
{
/* replaces each occurrence of ' ' in string "buf" with a null */

	char c;

	while (c = *buf) {
		if (c == ' ')
			*buf = '\0';
		++buf;
}	}

/*==================================================================*/

STATIC double
angle(string)
char *string;

/* Returns the radian value of the angle (degrees) in "string".
Leading nulls on "string" will be skipped, so it had better contain
at least one digit. */
{
	return atof(sknull(string)) * DE2RA;
}

/*==================================================================*/

STATIC char *
sknull(cptr)
char *cptr;
{
/* Skip nulls.  Returns pointer to first non-null char in "cptr". */

	while (!*cptr)
		++cptr;

	return cptr;
}

/*==================================================================*/

STATIC double
sci(string)
char *string;

/* Converts the quasi scientific notation of the "Motion Dot Dot/6" or
"BSTAR" field to double. */
{
	char buf[12], *bufptr = buf;

	if (string[1] == '\0')	/* is field blank? */
		return 0.;

	/* get mantissa */
	if (*string == '-')
		*bufptr++ = '-';
	++string;		/* point to 1st digit of mantissa */
	*bufptr++ = '.';
	strncpy(bufptr, string, 6);	/* mantissa has 5 digits */
	bufptr += 5;
	string += 5;
	*bufptr++ = 'E';

	/* get exponent */
	if (*string == '-')
		*bufptr++ = '-';	/* exponent sign, if neg. */

	*bufptr++ = *++string;		/* copy exponent */
	*bufptr = '\0';
	return atof(buf);
}
