/*
SEESAT.H
by Paul S. Hirose, Edwards AFB, Calif.
Declarations & definitions for globals in the SEESAT satellite tracking
program.
*/

/* printed out at program startup */
#define VERS "7 May 1989"

#include <ctype.h>
#include <stdio.h>
#include <math.h>

/* #defining MAIN before #including this header will generate
definitions.  If MAIN has not been #defined, only declarations will be
generated. */

#ifdef MAIN		/* generate DEFINITIONS */
#define CLASS
#define EQUALS(N) = N
#define DRIVER

#else			/* generate DECLARATIONS */
#define CLASS extern
#define EQUALS(N)
#endif

/*####################################################################
			MISC DEPENDENCIES
####################################################################*/

/* args to fopen() a file for ASCII read or write */
#define R "r"
#define W "w"

/* Some functions may be declared type "void".  If your compiler doesn't
support this keyword, "typedef int void;" or "#define void int" can be
inserted here to fix the problem. */

/* POW(x, y) raises x to the y power. */
#define POW(x, y) pow(x, y)
extern double pow();

extern double dint();

/* dint(3.6) == 3.0, dint(-3.6) == -3.0.
If your library has floor() and ceil() instead of dint(), retain the
preceding dint() declaration and un-comment the following code: */

#ifdef MAIN

double dint(x)
double x;
{
	if (x >= 0) return floor(x);
	return ceil(x);
}
#endif

/*####################################################################
			Mathematical Constants
####################################################################*/

CLASS double
	PI EQUALS(3.141592653589793),
	E6A EQUALS(1e-6),

	/* initialized by init() at program startup: */
	PIO2,		/* pi/2 */
	TOTHRD,		/* 2/3 */
	TWOPI,		/* 2pi */
	X3PIO2,		/* 3pi/2 */
	DE2RA,		/* radians per degree */
	RA2DE,		/* deg per radian */
	XJ3, XJ4;	/* 3rd & 4th gravitational zonal harmonic */

	/* XJ3 & XJ4 belong with the physical constants, but I initialize them
	at run-time because my compiler (Ecosoft C v. 3.5 for CP/M) won't take
	negative floating point initializers */


/*####################################################################
			Physical Constants
####################################################################*/

/* dimensions & gravity of earth, World Geodetic System 1972 values */

CLASS double
XJ2 EQUALS(1.082616e-3),	/* 2nd gravitational zonal harmonic */
				/* XJ3 & XJ4 are with the math constants */
XKE EQUALS(.743669161e-1),
XKMPER EQUALS(6378.135),	/* equatorial earth radius, km */
MEAN_R EQUALS(.998882406),	/* mean radius, in units of XKMPER */

/* misc */
XMNPDA EQUALS(1440.0),	/* time units/day */
AE EQUALS(1.0),		/* distance units, earth radii */
SIDSOL EQUALS(1.002737908);	/* sidereal/solar time ratio */


/*####################################################################
			Units & Conventions
####################################################################*/
/*
Unless the otherwise indicated, throughout this program
quantities are measured in the following units:

time (interval)		minutes
time (epoch)		minutes since 4713 B.C.
angle			radians
length			equatorial earth radii (1 unit = XKMPER km)

South declinations & south latitudes are negative.
Elevations below the horizon are negative.
East longitude is positive, west negative.
Azimuth is measured starting at north, increases as one turns
eastward, and 0 <= azimuth < 2pi radians.
*/


/*####################################################################
				Structures
####################################################################*/

/* rectangular coordinates */

struct xyz { double x, y, z; };


/* Spherical coordinates.  R is used for a linear measurement (e.g.,
radius), phi for an angle measured with respect to the principal
plane (e.g., latitude), and lambda for an angle measured IN the
principal plane (e.g., longitude). */

struct sph { double r, phi, lambda; };

/*####################################################################
				Data
####################################################################*/

/* The satellite's mean orbital elements.  These are read from the
orbital element file, and remain constant during program execution. */
CLASS double
	xmo,	/* mean anomaly */
	xnodeo,	/* right ascension of ascending node */
	omegao,	/* argument of the perigee */
	eo,	/* eccentricity */
	xincl,	/* inclination */
	xno,	/* mean motion, radians/min */
	xndt2o,	/* 1st time derivative of mean motion, or ballistic
		coefficient (depending on ephemeris type) */
	xndd6o, /* 2nd time derivative of mean motion */
	bstar,	/* BSTAR drag term if GP4 theory was used;
		otherwise, radiation pressure coefficient */
	epoch;	/* epoch of mean elements */

CLASS char name[12];	/* of satellite - also from element file */

/* Set by thetag(), used only in thetag() & deep space initialization. */
CLASS double ds50;

/* satellite geocentric coordinates output by sgp4() */
CLASS struct xyz sat;

/* satellite topocentric coordinates.  These all come from xyztop(). */
CLASS struct sph
	azel,	/* slant range, azimuth, elevation */
	radec,	/* slant range, R.A., dec. */
	latlon; /* altitude, latitude, longitude */

/* elevation (rounded to the nearest int) of center of sun above the
satellite's horizon.  Adjusted for dip due to height of satellite, but no
allowance for refraction.  Value comes from xyztop(). */
CLASS int elsusa;

/* Constants set at program startup by init(), and used by sgp4() */
CLASS double qoms2t, s, ck2, ck4;


/*####################################################################
				Functions
####################################################################*/

/* module READEL:  gets orbital elements from file */

extern int getel();	/* (fp)
FILE *fp;

Loads the mean orbital element variables with data from fp.  Initializes
precession by calling inpre() with the epoch of the orbital elements. 
Returns 1 if period of satellite >= 225 minutes, 0 if period < 225 min., -1
if error (e.g., checksum or format bad). */

/*==================================================================*/

/* module ASTRO:  astronomical time, coordinate transformation */

extern double julday();		/* (year, month, day)
int year, month, day;
Returns the Julian Day (measured in days, not minutes) at 0h UT on the
given date, Gregorian calendar.  No error checking for illegal dates. */


extern int xyztop();	/* (iflag2, tp)
int *iflag2;
double tp;		epoch, used to compute the sun's position

Satellite elevation is computed, based on coordinates in struct sat. 
If elevation is less than zero, xyztop() immediately returns 0; in
this case, elsusa, radec, azel, and latlon are undefined.  On the
other hand, if elevation is 0 or more, 1 is returned and the output
variables are filled in.

If *iflag2 == 1, xyztop() will assume that subsequent calls will have tp at
equally spaced intervals.  This saves the trouble of recomputing sidereal time
from scratch with each call.  *iflag2 is reset to 0 on second call. */


extern int dusk();	/* (ep)
double ep;		epoch

Returns elevation of sun (in degrees) at observer. */


extern int parall();		/* (ep)
double ep;			epoch

Returns the parallactic angle in degrees: the angle on the celestial sphere
whose vertex is the satellite, initial side contains the north pole, and
terminal side contains the zenith of the observer.  Angle increases east. */


extern double topos();	/* no args

Responsible for filling in all data in struct "obs" in module ASTRO.  Must be
called when SEESAT is started up, before any predictions are run.  Returns
zone description (local time - Greenwich time) in minutes. */


extern void inpre();	/* (ep)
double ep;

Sets up struct "dircos" (direction cosines) in module ASTRO to precess
satellite R.A/dec.  Initial epoch will be "ep" and final epoch is
determined by REFEP in ASTRO. */


extern double fmod2p();	/* (x)
double x;

Returns x, such that 0 <= x < 2pi, i.e., takes out multiples of 2pi. */


extern double thetag();	/* (ep)

Returns Greenwich hour angle of Aries at epoch ep.
Side effect: sets ds50 to minutes between 1950 Jan 0 0h UT & ep. */

/*==================================================================*/

/* module SGP4 */

extern void sgp4();	/* (iflag, tsince)
int *iflag;		signals first call in prediction run
double tsince;		(prediction epoch) - (epoch of mean elements)

The key function in the prediction program.  Returns the satellite's
position (at tsince after the epoch of its elements) in the global
struct "sat".  Make *iflag non-zero for the first call of sgp4() after
a new set of elements are loaded.  sgp4() will automatically reset
*iflag to 0. */


/*==================================================================*/

/* module DRIVE2 */

extern char **tok();	/* (argv)
char **argv;

If argv[0] == NULL, prints a > prompt, accepts input from keyboard. 
The entered string is converted to a series of sub-strings in upper
case.  An internal static array of char pointers to the sub-strings is
initialized.  Returns start address of the array.  E.g., if "step 10
run" was typed and p = the returned value, p[0] == "STEP", p[1] ==
"10", p[2] == "RUN", p[3] == NULL.

If argv[0] != NULL, tok() returns argv unchanged. */


extern double din();	/* (def)
char *def;

Input a double, with default value.  Sends '[', the "def" string, then
"]: " to the console.  E.g., "[2.718]: ".  Then waits for user input. 
If a only a carriage return is struck, returns "def" converted to a
double.  If a string is typed, returns the string converted to double. 
Backspacing is allowed.  "def" is passed as a string to allow complete
control over how it's presented to the user (rounding etc.). */


extern double atomin();	/* (string)
char *string;

Converts a string of form "hhmm:ss" to minutes. */


extern void tokdat();	/* (dmy, dest)
int *dmy;
char *dest;

Convert an array of ints to a date string.  For example, suppose dmy[]
contains {12, 4, 1987}.  A call of tokdat() will place "12 APR 1987"
in string "dest". */


extern void degdms();	/* (pre, x)
int pre;
double x;
writes to external struct "timstr"

Converts x degrees (or hours) to 3 ASCII strings holding degrees,
minutes, and seconds.  "Pre" specifies the desired rounding:  1 for
nearest 10 min, 2 for nearest 1 min, 3 for 10 sec, etc.  The strings
will be returned in the external struct "timstr". */


extern char *dmstim();	/* no args

Converts the strings in external struct "timstr" to a string of
"hhmm:ss" format.  Will be padded with zeros to fill out all the
digits.  Returns the string. */


extern int *dday();	/* (pargv)
char ***pargv;

"default day".  Processes the day, month, and year you enter as part
of the START & STOP commands.  Remembers the last values entered and
supplies them as defaults if you leave them out.  Returns day, month,
year in an array.  As a side effect, uses pargv to step the command
line pointer forward to the next command. */


extern void mjog();	/* no args
types a list of available commands to the console to jog your memory */


extern void prnval();	/* (tph)
double tph;

Prints one line of prediction data on console. */


extern char *skip0();	/* (string)
char *string;
Returns pointer to first char in string that's not '0'. */

/*##################################################################*/


#ifdef DRIVER

/*========================== DRIVER DATA ==========================*/
/* This data is only used by DRIVE1 & DRIVE2 modules. */

/* Size of this program's command line buffer (the program's
internal command line, not the operating system command line used to
start the program), and maximum number of tokens (space-delimited data
items) allowed on the command line.  */
#define LCLIN 100
#define NTOK 20

/* degdms() returns its strings here */
CLASS struct {
	char deg[5], min[3], sec[20];
} timstr;

#endif		/* end of DRIVE1 & DRIVE2 stuff */


/*####################################################################
			DEBUGGING FUNCTIONS
####################################################################*/
/* DEBUG is controlled in the file that #includes this file. */

#ifdef DEBUG
#define DTEST(a) ddebug a
#define ETEST(a) edebug a
#define FTEST(a) fdebug a
extern void ddebug(), edebug(), fdebug();

#ifdef MAIN

/* Debugging functions.  This stuff is only activated when
SEESAT.C is compiled with DEBUG on.  Each function printf()s a
variable number of arguments of a particular type.  If the bp
argument passed to the function matches variable stopat, the
arguments are printf() all on the same line.  If bp != stopat, no
action occurs.  The arguments to be printed are passed by
reference, and the number of arguments is given by argc. */

static int stopat;		/* currently active break point */

void
ddebug(bp, argc, arg1)
int bp, argc, *arg1;
{
	if (bp == stopat) {
		int **argptr;
		printf("%d: ", bp);
		for (argptr = &arg1; argc ; ++argptr, --argc)
			printf("%d ", **argptr);
		getbp();
}	}

void
edebug(bp, argc, prec, arg1)
int bp, argc, prec;
double *arg1;
{
	if (bp == stopat) {
		double **argptr;
		printf("%d: ", bp);
		for (argptr = &arg1; argc; ++argptr, --argc)
			printf("%.*e ", prec, **argptr);
		getbp();
}	}

void
fdebug(bp, argc, prec, arg1)
int bp, argc, prec;
double *arg1;
{
	if (bp == stopat) {
		double **argptr;
		printf("%d: ", bp);
		for (argptr = &arg1; argc; ++argptr, --argc)
			printf("%.*f ", prec, **argptr);
		getbp();
}	}

static void
getbp()

/* ask for next breakpoint, put value in stopat */
{
	char buffer[7];
	printf("\nnext breakpoint? ");
	stopat = atoi(gets(buffer));
}

#endif

#else
/* Debugging is turned off, so expand the macros to nothing */
#define DTEST(a)
#define ETEST(a)
#define FTEST(a)

#endif
